#ifdef GUITEST
#include <stdlib.h>
#include <string.h>
#include "main.h"
#include "gui.h"

class GUIWindow;

typedef struct __ChildWindow
{
 GUIWindow *window;
 uint32 x,y;
 struct __ChildWindow *next;
} ChildWindow;

class GUIWindow
{
	friend class GUIList;
	public:

	GUIWindow(void);
	~GUIWindow(void);

	bool Attach(uint32 x, uint32 y, GUIWindow *);
	virtual void Draw(uint8 *dest);

	private:
	ChildWindow *children;
	uint32 width;
	uint32 height;
};

class ListEntry
{
	friend class GUIList;
	UTF8 *text;
	void *data;
	ListEntry *next;

	ListEntry(UTF8 *ntext, void *ndata)
	{
	 text = ntext;
	 data = ndata;
	}
};

class GUIList: public GUIWindow
{
	friend class ListEntry;
	public:
	GUIList(void);
	void Draw(uint8 *dest);
	bool AppendItem(UTF8 *text, void *data);

	private:
	ListEntry *ListItems;
	uint32 ListCount;
};

void DrawTextTrans(uint8 *dest, uint32 width, uint8 *msg, uint8 fgcolor, int centered);

static void DrawText(uint8 *dest, uint32 width, int xpos, int ypos, uint8 *msg, uint8 fgcolor, int centered)
{
 DrawTextTrans(dest + xpos + ypos * 256, width, msg, fgcolor, centered);
}

static void DrawRectangle(uint8 *dest, int pitch, int xpos, int ypos, int width, int height, int color)
{
 int x;
 int y;

 dest += pitch * ypos + xpos;
 /* Draw top and bottom horiz */
 for(x=0;x<width;x++)
 {
  dest[x] = color;
  dest[x + (height-1) * pitch] = color;
 }

 for(y=0;y<height;y++)
 {
  dest[y * pitch] = color;
  dest[y * pitch + width-1] = color;
 }
}

static void DrawRectangleFill(uint8 *dest, int pitch, int xpos, int ypos, int width, int height, int color, int fillcolor)
{
 int x;
 int y;

 dest += pitch * ypos + xpos;
 /* Draw top and bottom horiz */
 for(x=0;x<width;x++)
 {
  dest[x] = color;
  dest[x + (height-1) * pitch] = color;
 }

 for(y=0;y<height;y++)
 {
  dest[y * pitch] = color;
  dest[y * pitch + width-1] = color;
 }

 /* Now draw inner area. */
 dest += 1 + 1 * pitch;

 for(y=0;y<height-2;y++)
  for(x=0;x<width-2;x++)
  {
   dest[y*pitch+x]=fillcolor;
  }
}

bool GUIWindow::Attach(uint32 x, uint32 y, GUIWindow *window)
{
 children = (ChildWindow *)malloc(sizeof(ChildWindow));

 children->next = 0;
 children->x = x;
 children->y = y;
 children->window = window;

 return(1);
}

GUIWindow::GUIWindow(void)
{
 puts("GUI Constructor");
}

void GUIWindow::Draw(uint8 *dest)
{
 children->window->Draw(dest);
}

GUIList::GUIList(void)
{
 ListCount = 0;
 ListItems = NULL;
}

void GUIList::Draw(uint8 *dest)
{
//static void 
// DrawRectangleFill(dest, 256, 0, 0, width, height, 1, 4);
 //DrawRectangle(
}

bool GUIList::AppendItem(UTF8 *text, void *data)
{
 ListEntry *curListEntry;
 ListEntry *newListEntry;

 if(!ListItems)
  curListEntry = 0;
 else
 {
  curListEntry = ListItems;
  while(curListEntry->next)
   curListEntry = curListEntry->next;
 }

 text = (UTF8 *)strdup((char *)text);
 newListEntry = new ListEntry(text, data);
 ListCount++;
 return(1);
}


static GUIWindow *mw;
static GUIList *glist;

void DrawGUI(uint8 *XBuf)
{
 if(!XBuf) return;

 if(!mw)
 {
  mw = new GUIWindow();
  glist = new GUIList();
 
  glist->AppendItem((UTF8 *)"BEEEEEP", 0);  
  mw->Attach(0, 0, glist);
  mw->Draw(XBuf);
  //fw->Draw(XBuf);
 }
 //DrawRectangleFill(XBuf, 256, 0, 8, 256, 15, 4, 0);

 //DrawRectangle(XBuf, 256, 0, 22, 256, 232 - 22, 4);
 //DrawText(XBuf, 256, 2, 9, (UTF8*)"Games Audio Video Input", 4, 0);
}
#endif
